
// Flutter BLE App to scan and read freshness value from SmartSpoilCompact

import 'package:flutter/material.dart';
import 'package:flutter_blue/flutter_blue.dart';

void main() => runApp(FreshnessApp());

class FreshnessApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Food Freshness Tracker',
      theme: ThemeData(primarySwatch: Colors.green),
      home: FreshnessHomePage(),
    );
  }
}

class FreshnessHomePage extends StatefulWidget {
  @override
  _FreshnessHomePageState createState() => _FreshnessHomePageState();
}

class _FreshnessHomePageState extends State<FreshnessHomePage> {
  BluetoothDevice? targetDevice;
  BluetoothCharacteristic? freshnessCharacteristic;
  String freshnessValue = "Not Connected";

  final String targetDeviceName = "SmartSpoilCompact";
  final Guid serviceUUID = Guid("4fafc201-1fb5-459e-8fcc-c5c9c331914b");
  final Guid characteristicUUID = Guid("beb5483e-36e1-4688-b7f5-ea07361b26a8");

  @override
  void initState() {
    super.initState();
    scanForDevice();
  }

  void scanForDevice() {
    FlutterBlue.instance.startScan(timeout: Duration(seconds: 4));
    FlutterBlue.instance.scanResults.listen((results) {
      for (ScanResult r in results) {
        if (r.device.name == targetDeviceName) {
          targetDevice = r.device;
          connectToDevice();
          FlutterBlue.instance.stopScan();
          break;
        }
      }
    });
  }

  void connectToDevice() async {
    if (targetDevice == null) return;

    await targetDevice!.connect();
    discoverServices();
  }

  void discoverServices() async {
    if (targetDevice == null) return;

    List<BluetoothService> services = await targetDevice!.discoverServices();
    for (BluetoothService service in services) {
      if (service.uuid == serviceUUID) {
        for (BluetoothCharacteristic c in service.characteristics) {
          if (c.uuid == characteristicUUID) {
            freshnessCharacteristic = c;
            await c.setNotifyValue(true);
            c.value.listen((value) {
              setState(() {
                freshnessValue = String.fromCharCodes(value);
              });
            });
          }
        }
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: Text("Freshness Tracker")),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Icon(Icons.eco, size: 100, color: Colors.green),
            SizedBox(height: 20),
            Text("Current Freshness Status:", style: TextStyle(fontSize: 20)),
            SizedBox(height: 10),
            Text(freshnessValue, style: TextStyle(fontSize: 24, fontWeight: FontWeight.bold)),
            SizedBox(height: 40),
            ElevatedButton(
              onPressed: () => scanForDevice(),
              child: Text("Rescan Device"),
            )
          ],
        ),
      ),
    );
  }
}
